/****************************************************************************

    GUIMEM --

    GUI version of character based Memory display program.

    Version 1.00

    David A. Feinleib                   3/19/92

****************************************************************************/

#include <windows.h>                /* required for all Windows applications */
#include <stdlib.h>
#include "guimem.h"                /* specific to this program              */

HANDLE hInst;                       /* current instance                      */

BOOL FAR PASCAL DispMem(HWND, UINT, UINT, LONG);

/****************************************************************************


****************************************************************************/

int APIENTRY WinMain(
    HANDLE hInstance,
    HANDLE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
    )
{

    MSG msg;                                 /* message                      */
    FARPROC lpProc;

    UNREFERENCED_PARAMETER( lpCmdLine );

    hInst = hInstance;
    lpProc = MakeProcInstance((FARPROC)DispMem, hInst);

    DialogBox(hInst,                    /* current instance         */
		"DispMem",               /* resource to use          */
		NULL,                   /* parent handle            */
		(WNDPROC)lpProc);  /* About() instance address */
    FreeProcInstance(lpProc);

    return 0;
    if (!hPrevInstance)                  /* Other instances of app running? */
	if (!InitApplication(hInstance)) /* Initialize shared things */
	    return (FALSE);              /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow))
	return (FALSE);

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg,        /* message structure                      */
	    NULL,                  /* handle of window receiving the message */
	    NULL,                  /* lowest message to examine              */
	    NULL))                 /* highest message to examine             */
	{
	TranslateMessage(&msg);    /* Translates virtual key codes           */
	DispatchMessage(&msg);     /* Dispatches message to window           */
    }
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

****************************************************************************/

BOOL InitApplication(HANDLE hInstance)       /* current instance             */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style = NULL;                    /* Class style(s).                    */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;       /* Function to retrieve messages for  */
					/* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance = hInstance;          /* Application that owns the class.   */
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName =  "guimemMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "guimemWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass(&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

	
****************************************************************************/

BOOL InitInstance(
    HANDLE          hInstance,          /* Current instance identifier.       */
    int             nCmdShow)           /* Param for first ShowWindow() call. */
{
    HWND            hWnd;               /* Main window handle.                */

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow(
	"guimemWClass",                /* See RegisterClass() call.          */
	"guimem Sample Application",   /* Text for window title bar.         */
	WS_OVERLAPPEDWINDOW,            /* Window style.                      */
	CW_USEDEFAULT,                  /* Default horizontal position.       */
	CW_USEDEFAULT,                  /* Default vertical position.         */
	CW_USEDEFAULT,                  /* Default width.                     */
	CW_USEDEFAULT,                  /* Default height.                    */
	NULL,                           /* Overlapped windows have no parent. */
	NULL,                           /* Use the window class menu.         */
	hInstance,                      /* This instance owns this window.    */
	NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
	return (FALSE);

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow(hWnd, nCmdShow);  /* Show the window                        */
    UpdateWindow(hWnd);          /* Sends WM_PAINT message                 */
    return (TRUE);               /* Returns the value from PostQuitMessage */

}

/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

****************************************************************************/

LONG APIENTRY MainWndProc(
	HWND hWnd,                /* window handle                   */
	UINT message,             /* type of message                 */
	UINT wParam,              /* additional information          */
	LONG lParam)              /* additional information          */
{
    FARPROC lpProcAbout;          /* pointer to the "About" function */

    switch (message) {
	case WM_COMMAND:           /* message: command from application menu */
	    if (LOWORD(wParam) == IDM_ABOUT) {
		    lpProcAbout = MakeProcInstance((FARPROC)About, hInst);

		DialogBox(hInst,                 /* current instance         */
		    "AboutBox",                  /* resource to use          */
			hWnd,                    /* parent handle            */
			(WNDPROC)lpProcAbout);           /* About() instance address */

		FreeProcInstance(lpProcAbout);
		break;
	    }
	    else                            /* Lets Windows process it       */
		    return (DefWindowProc(hWnd, message, wParam, lParam));

	case WM_DESTROY:                  /* message: window being destroyed */
	    PostQuitMessage(0);
	    break;

	default:                          /* Passes it on if unproccessed    */
	    return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (NULL);
}


/****************************************************************************

    FUNCTION: About(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box


****************************************************************************/

BOOL APIENTRY About(
	HWND hDlg,                /* window handle of the dialog box */
	UINT message,             /* type of message                 */
	UINT wParam,            /* message-specific information    */
	LONG lParam)
{
    switch (message) {
	case WM_INITDIALOG:                /* message: initialize dialog box */
	    return (TRUE);

	case WM_COMMAND:                      /* message: received a command */
	    if (LOWORD(wParam) == IDOK          /* "OK" box selected?        */
		|| LOWORD(wParam) == IDCANCEL) {        /* System menu close command? */
		EndDialog(hDlg, TRUE);        /* Exits the dialog box        */
		return (TRUE);
	    }
	    break;
    }
    return (FALSE);                           /* Didn't process a message    */
	UNREFERENCED_PARAMETER(lParam);
}

BOOL APIENTRY DispMem(
	HWND hDlg,                /* window handle of the dialog box */
	UINT message,             /* type of message                 */
	UINT wParam,            /* message-specific information    */
	LONG lParam)
{
    MEMORYSTATUS ms;
    char szBuf[129];

    switch (message) {
	case WM_INITDIALOG:                /* message: initialize dialog box */
	    ms.dwLength = sizeof(MEMORYSTATUS);
	    GlobalMemoryStatus(&ms);

	    ltoa(ms.dwMemoryLoad, szBuf, 10);        
	    SetDlgItemText(hDlg, 200, szBuf);

	    ltoa(ms.dwAvailPhys, szBuf, 10);
	    SetDlgItemText(hDlg, 201, szBuf);

	    ltoa(ms.dwTotalPhys, szBuf, 10);
	    SetDlgItemText(hDlg, 301, szBuf);

	    ltoa(ms.dwAvailPageFile, szBuf, 10);
	    SetDlgItemText(hDlg, 202, szBuf);

	    ltoa(ms.dwTotalPageFile, szBuf, 10);
	    SetDlgItemText(hDlg, 302, szBuf);

	    ltoa(ms.dwAvailVirtual, szBuf, 10);
	    SetDlgItemText(hDlg, 203, szBuf);

	    ltoa(ms.dwTotalVirtual, szBuf, 10);
	    SetDlgItemText(hDlg, 303, szBuf);
	    return (TRUE);

	case WM_COMMAND:                      /* message: received a command */
	    if (LOWORD(wParam) == IDOK          /* "OK" box selected?        */
		|| LOWORD(wParam) == IDCANCEL) {        /* System menu close command? */
		EndDialog(hDlg, TRUE);        /* Exits the dialog box        */
		return (TRUE);
	    }
	    break;
    }
    return (FALSE);                           /* Didn't process a message    */
	UNREFERENCED_PARAMETER(lParam);
}
