/**************************************************************************\
*  locale.c -- sample program demonstrating the locale... APIs
*
*  In this sample the main window is a dialog box.  There is no need to
*   register a new window class or create a new window.  Instead just call
*   DialogBox() and use the template defined in the .RC file.  All of the
*   interesting code is thus in the window procedure for the dialog box.
*   In this case, simply respond to the combobox command messsages and fill
*   the list box with appropriate values.
*
\**************************************************************************/

#define UNICODE


#include <windows.h>
#include "locale.h"


/**************************************************************************\
*
*  function:  WinMain()
*
*  input parameters:  c.f. generic sample
*
\**************************************************************************/
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine, int nCmdShow)
{
int ret;

    UNREFERENCED_PARAMETER( hPrevInstance );
    UNREFERENCED_PARAMETER( lpCmdLine );
    UNREFERENCED_PARAMETER( nCmdShow);

    ret = DialogBox (hInstance, TEXT("localeDlg"), NULL, (DLGPROC)MainDlgProc);
    return ret;
}



/**************************************************************************\
*
*  function:  MainDlgProc()
*
*  input parameters:  standard window procedure parameters.
*
\**************************************************************************/
LRESULT MainDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
int i;
TCHAR buffer[MAXTCHAR];

  UNREFERENCED_PARAMETER(lParam);


  switch (message) {

    /********************************************************************\
    * WM_SYSCOMMAND
    *
    * ignore all syscommand messages, except for SC_CLOSE.
    *  on this one, call EndDialog().
    \********************************************************************/
    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE) {
        EndDialog (hwnd, TRUE);
        return TRUE;
      } else
        return FALSE;
    break;

    /********************************************************************\
    * WM_INITDIALOG
    *
    * Fill the combobox with the language, sublanguage pairs.
    *
    \********************************************************************/
    case WM_INITDIALOG:
      for (i = 0; i < NLANG; i++) {

        SendDlgItemMessage (hwnd, DID_COMBOBOX, CB_ADDSTRING, 0,
            (LPARAM)langstrings[i].String);
      }
//      SendDlgItemMessage (hwnd, DID_COMBOBOX, CB_SHOWDROPDOWN, (WPARAM) TRUE, 0);
    return NULL;



    /********************************************************************\
    * WM_COMMAND
    *
    * When the different languages are selected, clear the list box,
    *  then fill it up again with the new information.
    *
    \********************************************************************/
    case WM_COMMAND:
      if ((HIWORD(wParam) == CBN_SELCHANGE) && (LOWORD(wParam) == DID_COMBOBOX)) {
        int iSel;
        TCHAR linfo[MAXTCHAR];

        iSel = SendMessage ((HWND) lParam, CB_GETCURSEL, 0,0);
        SendDlgItemMessage (hwnd, DID_LISTBOX, LB_RESETCONTENT, 0, 0);

        for (i = 0; i < NLOCALEENTRIES; i++) {
          linfo[0] = 0;
          GetLocaleInfoW (MAKELCID (langstrings[iSel].Value),
                  localeentry[i].Value, linfo, MAXTCHAR);

          wsprintf (buffer, " %ts \t %ts", localeentry[i].String, linfo);

          SendDlgItemMessage (hwnd, DID_LISTBOX, LB_ADDSTRING, 0, (LPARAM)buffer);
        }

      }
    break; /* end WM_COMMAND */


    default: return FALSE;
  } /* end switch(message) */
}
