// portio.c  -  Simple routines for using RAWIO device

#include <windows.h>
#include "portio.h"

// *****************************************************
// The following private data and procedures should NOT 
// be accessed directly by application code. Use the
// public procedures which appear later and have
// prototypes defined in portio.h.
// *****************************************************

// define handle for RAWIO device

static HANDLE hPrivateRawIO = INVALID_HANDLE_VALUE;

// define actual procedures for doing port I/O

static VOID OpenPrivateRawIO(VOID)
{
    hPrivateRawIO = CreateFile(
                     "\\\\.\\RAWIO",
                     GENERIC_READ | GENERIC_WRITE,
                     0,
                     NULL,
                     OPEN_EXISTING,
                     FILE_ATTRIBUTE_NORMAL,
                     NULL
                   );		  
}

// *****************************************************
// The following public procedures can all be accessed 
// directly by application code. In addition, application
// code can safely use the handle returned by
// GetRawIOHandle() in custom calls to DeviceIoControl()
// to write or read blocks of port data. The application
// SHOULD NOT close the handle unless it is finished
// with all port i/o operations. The application need not 
// ever close the handle since the RAWIO device is not
// exclusive (multiple processes can access it 
// simultaneously) and since the handle will be closed
// automatically on task exit. The prototypes for these
// are in portio.h.
// *****************************************************

VOID outp(WORD port, BYTE value)
{
    DWORD obuffer[2];
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer[0] = (DWORD)port;
    obuffer[1] = (DWORD)value;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_OUT_BYTE,
	    obuffer,
	    8,
	    NULL,
	    0,
	    &i,
	    NULL
	    );
}

VOID outpw(WORD port, WORD value)
{
    DWORD obuffer[2];
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer[0] = (DWORD)port;
    obuffer[1] = (DWORD)value;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_OUT_WORD,
	    obuffer,
	    8,
	    NULL,
	    0,
	    &i,
	    NULL
	    );
}

VOID outpl(WORD port, DWORD value)
{
    DWORD obuffer[2];
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer[0] = (DWORD)port;
    obuffer[1] = (DWORD)value;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_OUT_DWORD,
	    obuffer,
	    8,
	    NULL,
	    0,
	    &i,
	    NULL
	    );
}

BYTE inp(WORD port)
{
    DWORD obuffer;
    DWORD ibuffer;	
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer = (DWORD)port;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_IN_BYTE,
	    &obuffer,
	    4,
	    &ibuffer,
	    4,
	    &i,
	    NULL
	    );

    return((BYTE)ibuffer);
}

WORD inpw(WORD port)
{
    DWORD obuffer;
    DWORD ibuffer;	
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer = (DWORD)port;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_IN_WORD,
	    &obuffer,
	    4,
	    &ibuffer,
	    4,
	    &i,
	    NULL
	    );

    return((WORD)ibuffer);
}

DWORD inpl(WORD port)
{
    DWORD obuffer;
    DWORD ibuffer;	
    LONG i;
    
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    
    obuffer = (DWORD)port;
    
    DeviceIoControl(
	    hPrivateRawIO,
	    IOCTL_RAWIO_IN_DWORD,
	    &obuffer,
	    4,
	    &ibuffer,
	    4,
	    &i,
	    NULL
	    );

    return((DWORD)ibuffer);
}

HANDLE GetRawIoHandle(VOID)
{
    if (hPrivateRawIO == INVALID_HANDLE_VALUE) OpenPrivateRawIO();
    return hPrivateRawIO;
}

