// portio.h  -  Include file for using RAWIO device

// *****************************************************
//
// Define prototypes for application callable procedures
//
// *****************************************************

// i386/486 port output (8, 16, or 32 bits)

VOID outp(WORD port, BYTE value);
VOID outpw(WORD port, WORD value);
VOID outpl(WORD port, DWORD value);

// i386/486 port input (8, 16, or 32 bits)

BYTE inp(WORD port);
WORD inpw(WORD port);
DWORD inpl(WORD port);

// get file handle for RAWIO logical device

HANDLE GetRawIoHandle(void);


// **************************************************
//
// The rest of this defines the private I/O control
// function codes used by the RAWIO device driver.
// An application only needs these if it intends to
// talk directly to the driver through DeviceIoControl()
// operations.
//
// **************************************************

//
// Macro definition for defining IOCTL function control codes.  Note
// that function codes 0-2047 are reserved for Microsoft Corporation, and
// 2048-4095 are reserved for customers.
//

#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)

//
// Define the method codes for how buffers are passed for I/O and FS controls
//

#define METHOD_BUFFERED                 0
#define METHOD_IN_DIRECT                1
#define METHOD_OUT_DIRECT               2
#define METHOD_NEITHER                  3

//
// Define the access check value for any access
//
//
// The FILE_READ_ACCESS and FILE_WRITE_ACCESS constants are also defined in
// ntioapi.h as FILE_READ_DATA and FILE_WRITE_DATA. The values for these
// constants *MUST* always be in sync.
//

#define FILE_ANY_ACCESS                 0
#define FILE_READ_ACCESS          ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS         ( 0x0002 )    // file & pipe

// a device type > 32767 selected for the raw I/O device

#define FILE_DEVICE_RAW_IO       0xFFF0

// define the IO control function codes

#define IOCTL_RAWIO_OUT_BYTE \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f0,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)
#define IOCTL_RAWIO_OUT_WORD \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f1,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)
#define IOCTL_RAWIO_OUT_DWORD \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f2,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)
#define IOCTL_RAWIO_IN_BYTE  \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f3,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)
#define IOCTL_RAWIO_IN_WORD  \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f4,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)
#define IOCTL_RAWIO_IN_DWORD  \
    0x80000000 | CTL_CODE(FILE_DEVICE_RAW_IO,0x08f5,\
    METHOD_BUFFERED,FILE_READ_ACCESS|FILE_WRITE_ACCESS)

